@@ -32,29 +32,29 @@ __all__ = [
 	'JSet',
 	# .addressing
 	'IPv4', 'IPv6', 'addressing', 'Validation', 'get_summaries', 'isSubset',
-	'binsubnet', 'bin2dec', 'bin2decmask', 'to_dec_mask', 'bin_mask',
+	'binsubnet', 'bin2dec', 'bin2decmask', 'to_dec_mask', 'bin_mask', 'Routes',
 	# .gpl
 	'Default', 'Container', 'Numeric', 'DifferenceDict', 
 	'STR', 'IO', 'LST', 'DIC', 'LOG', 'DB', 'IP', 'XL_READ', 'XL_WRITE', 
-	'DictMethods', 'Multi_Execution',
+	'DictMethods', 'Multi_Execution', 'nslookup'
 	# .convertdict
 	'ConvDict',
 	]
 
-__version__ = "0.0.9"
+__version__ = "0.0.10"
 
 from .juniper import Juniper
 from .jset import JSet
 
 from .addressing import (
 	IPv4, IPv6, addressing, Validation, get_summaries, isSubset,
-	binsubnet, bin2dec, bin2decmask, to_dec_mask, bin_mask,
+	binsubnet, bin2dec, bin2decmask, to_dec_mask, bin_mask, Routes,
 	)
 
 from .gpl import (Default, Container, Numeric, 
 	DifferenceDict, DictMethods, DIC,
 	STR, IO, LST, LOG, DB, IP, XL_READ, XL_WRITE, 
-	Multi_Execution,
+	Multi_Execution, nslookup
 	)
 
 from .convertdict import ConvDict
\ No newline at end of file
